#! /bin/sh

# duplicate.sh
# a bash script to copy a Max External Project, renaming all the files and class names in the process
# usage : cd to the maxbuild folder and type sh duplicate.sh [input] [output] (no slashes)
# by oli larkin www.olilarkin.co.uk

# TODO:
# subsitute other chars that you wouldn't want in a c file
# maybe better to subsitute tildes for word tilde inside c/cpp files

# NOTES:
# best not put any non alpha-numeric characters in your project names, except tildes and dots

# remove the forward slash if it's there
IN=${1//\/} 
OUT="$2"

# remove any underscores or hyphens
IN=${IN//\_}
IN=${IN//\-} 
OUT=${OUT//\_}
OUT=${OUT//\-} 

#remove tildes for substitutions inside code
CODEIN=${IN//\~}
CODEOUT=${OUT//\~} 

#remove dots for substitutions inside code
CODEIN=${CODEIN//\.}
CODEOUT=${CODEOUT//\.} 

#echo "Input project name: $IN"
#echo "Output project name: $OUT"

if [ -d "$OUT" ]; then
	echo "A project folder with the name you specified already exits, please choose a different name"
	exit
fi

#echo "creating folder..."

#copy entire $IN folder to $OUT
cp -R "$IN/" $OUT

#delete some folders and files we know we don't want, if they exist
if [ -d "$OUT/.svn" ]; then 
	rm -R "$OUT/.svn" 
fi

if [ -f "$OUT/$IN.ncb" ]; then 
	rm "$OUT/$IN.ncb" 
fi

if [ -d "$OUT/build-osx" ]; then 
	rm -R "$OUT/build-osx" 
fi

if [ -d "$OUT/build-win" ]; then 
	rm -R "$OUT/build-win" 
fi

if [ -d "$OUT/output/$IN.mxo" ]; then 
	rm -R "$OUT/output/$IN.mxo" 
fi

if [ -f "$OUT/output/$IN.mxe" ]; then 
	rm "$OUT/output/$IN.mxe" 
fi

#make an empty folder for the xcode project
mkdir "$OUT/$OUT.xcodeproj"

#echo "finding and replacing strings..."

#search for files whose contents contain the search input name search string $IN or $CODEIN (without dots and tildes)
for file in $(grep -l -R -e "$IN" -e "$CODEIN" "$OUT") 
do

	# here follows some hacky lines to make sure that the $OUT path part of $file doesn't get substituted
	#remove $OUT from the front of $file
	subpath=${file#$OUT}
		
	#replace all instances of $IN with $OUT in $subpath
	subpath=${subpath//$IN/$OUT}
	
	#stick the $OUT back on the front
	outfile=$OUT$subpath
	
	#check if we're seding the main c or cpp file, so that tildes and dots don't end up in the c code
	if [ ${file} == "$OUT/$IN.c"* ]
	then
		# first search inside file and replace all occurences of $IN with $OUT (this gets the external names in quotes with dots and tildes)
		sed -i "" -e 's_'\"$IN\"'_'\"$OUT\"'_g' $file
		# search inside file and replace all occurences of $CODEIN with $CODEOUT
		sed -i "" -e 's_'$CODEIN'_'$CODEOUT'_g' $file
		
    else
		# search inside files and replace all occurences of $IN with $OUT
		sed -i "" -e 's_'$IN'_'$OUT'_g' $file
	fi
	
	mv $file $outfile
	echo "Copied and modified: " $outfile
	
done

# remove $IN xcode project (folder)
rm -R "$OUT/$IN.xcodeproj"

echo "done!"