/* 

	"mb.maxobex" - a basic max object with obex for pattr/attribute functionality

	www.olilarkin.co.uk

*/

#include "ext.h"		// Required for all Max external objects
#include "commonsyms.h"	// Common symbols used by the Max 4.5 API
#include "ext_obex.h"



typedef struct _mbmaxobex 	// Data structure for this object
{
	t_object ob;
	void *obex; 
	void *out;
	double myfloat;
} t_mbmaxobex;	

//prototypes for methods
void *mbmaxobex_new(t_symbol *s, long argc, t_atom *argv);
void mbmaxobex_free(t_mbmaxobex *x);
void mbmaxobex_assist(t_mbmaxobex *x, void *b, long m, long a, char *s);
//void mbmaxobex_anything(t_mbmaxobex *x, t_symbol *s, long argc, t_atom *argv);
void mbmaxobex_int(t_mbmaxobex *x, long c);
void mbmaxobex_bang(t_mbmaxobex *x);
void mbmaxobex_float(t_mbmaxobex *x, double v);
void mbmaxobex_output(t_mbmaxobex *x);

//custom attribute set method prototypes
t_max_err mbmaxobex_myfloat(t_mbmaxobex *x, void *attr, long ac, t_atom *av);
t_max_err mbmaxobex_myint(t_mbmaxobex *x, void *attr, long ac, t_atom *av);

// pattr support function prototypes
//t_max_err mbmaxobex_getvalueof(t_mbmaxobex *x, long *ac, t_atom **av);
//t_max_err mbmaxobex_setvalueof(t_mbmaxobex *x, long ac, t_atom *av);

void *mbmaxobex_class;

void main(void)
{	
	t_class	*c;
	void	*attr;
	long	attrflags = 0;
	
	c = class_new("mb.maxobex", (method)mbmaxobex_new, (method)mbmaxobex_free, (short)sizeof(t_mbmaxobex), 
				  0L, A_GIMME, 0);
	
	// initialize the common symbols, since we want to use them
	common_symbols_init();
	
	// register the byte offset of obex with the class
	class_obexoffset_set(c, calcoffset(t_mbmaxobex, obex));
	
	// add some attributes
	attr = attr_offset_new("myfloat", _sym_float64, attrflags,
						   (method)0L,(method)mbmaxobex_myfloat, calcoffset(t_mbmaxobex, myfloat));
//	attr_addfilter_clip(attr,0., 1., 1, 1); 
	class_addattr(c, attr);	
	
//	attr = attr_offset_new("myint", _sym_long, attrflags,
//						   (method)0L,(method)mbmaxobex_myint, calcoffset(t_mbmaxobex, myint));
//	attr_addfilter_clip(attr,0, 1, 1, 1); 
//	class_addattr(c, attr);	
	
	// add methods to the class
	class_addmethod(c, (method)mbmaxobex_bang,		"bang",		0);
	class_addmethod(c, (method)mbmaxobex_float,		"float",	A_FLOAT, 0);
	class_addmethod(c, (method)mbmaxobex_int,		"int",		A_LONG, 0);
//	class_addmethod(c, (method)mbmaxobex_anything,	"list",		A_GIMME,0);	
//	class_addmethod(c, (method)mbmaxobex_anything,	"anything",	A_GIMME,0);	
	class_addmethod(c, (method)mbmaxobex_assist, 	"assist",	A_CANT,0);  
	
	// these methods support the pattr set of objects
//	class_addmethod(c, (method)mbmaxobex_getvalueof, 		"getvalueof", 	A_CANT,0);  
//	class_addmethod(c, (method)mbmaxobex_setvalueof, 		"setvalueof", 	A_CANT,0);
	
	// add methods for dumpout and quickref	
//	class_addmethod(c, (method)object_obex_dumpout,		"dumpout",		A_CANT, 0); 
//	class_addmethod(c, (method)object_obex_quickref,	"quickref",		A_CANT, 0);
	
	// we want this class to instantiate inside of the Max UI; ergo CLASS_BOX
	class_register(CLASS_BOX, c);
	mbmaxobex_class = c;
}

void mbmaxobex_bang(t_mbmaxobex *x)
{
	mbmaxobex_output(x);
}

void mbmaxobex_float(t_mbmaxobex *x, double f)
{
	object_attr_setfloat(x, gensym("floatval"), f);
	mbmaxobex_output(x);
}

void mbmaxobex_int(t_mbmaxobex *x, long c)
{
	object_attr_setfloat(x, gensym("floatval"), (double) c);
	mbmaxobex_output(x);
}

void mbmaxobex_output(t_mbmaxobex *x)
{
	outlet_float(x->out, x->myfloat);
}

t_max_err mbmaxobex_myfloat(t_mbmaxobex *x, void *attr, long ac, t_atom *av)
{
	t_box *b;
	float val;
	
	if (ac && av)
	{
		val = atom_getfloat(av);
		
		x->myfloat = val;
		
		object_obex_lookup(x, gensym("#B"), (t_object **)&b);	
		mbmaxobex_output(x);
	}
	return MAX_ERR_NONE;
}

/*
t_max_err mbmaxobex_myint(t_mbmaxobex *x, void *attr, long ac, t_atom *av)
{
	t_box *b;
	long val;
	
	if (ac && av)
	{
		val = atom_getlong(av);
		
		x->myfloat = (int) val;
		
		object_obex_lookup(x, gensym("#B"), (t_object **)&b);
	}
	return MAX_ERR_NONE;
}
*/

void mbmaxobex_assist(t_mbmaxobex *x, void *b, long m, long a, char *s)
{
	if (m == 1) { //input
		sprintf(s, "messages");			
	} 
	else {	//output
		if (a == 0)
			sprintf(s, "first out"); 
		//else
		//	sprintf(s, "dumpout");
	}
}


void mbmaxobex_free(t_mbmaxobex *x)
{
 
}

void *mbmaxobex_new(t_symbol *s, long argc, t_atom *argv)
{
	t_mbmaxobex *x;
	
	// we use object_alloc here, rather than newobject
	if (x = (t_mbmaxobex *)object_alloc(mbmaxobex_class)) {
		
		// add a dumpout outlet
		//object_obex_store(x, _sym_dumpout, outlet_new(x, NULL));
		
		// add left outlet
		x->out = outlet_new(x, 0L);
		
		// set defaults before attribute args are processed!
		x->myfloat = 0.;
		
		attr_args_process(x, argc, argv);		
	}
	return (x);
}

/*
t_max_err mbmaxobex_getvalueof(t_mbmaxobex *x, long *ac, t_atom **av)
{
	if (ac && av) 
	{
	
	}
	return MAX_ERR_NONE;
}

t_max_err mbmaxobex_setvalueof(t_mbmaxobex *x, long ac, t_atom *av)
{
	if (ac && av) 
	{

	}
	return MAX_ERR_NONE;
}
*/
