/* 

	"mb.mspobex~" - a basic starting point msp external with obex, one signal input, one output

	www.olilarkin.co.uk

*/

#include "ext.h"    // Required for all Max external objects
#include "z_dsp.h"
#include "ext_obex.h" 
#include "commonsyms.h"				// Common symbols used by the Max 4.5 API

void *mbmspobex_class;

typedef struct _mbmspobex 	// Data structure for this object
{
	t_pxobject x_obj; 
	void *obex; 
	double myfloat;
	long myint;
} t_mbmspobex;	

//prototypes for methods
void *mbmspobex_new(t_symbol *s, short argc, t_atom *argv);
void mbmspobex_free(t_mbmspobex *x);	
t_int *mbmspobex_perform(t_int *w); // peform routine with one signal inlet connected
void mbmspobex_dsp(t_mbmspobex *x, t_signal **sp, short *count);
void mbmspobex_assist(t_mbmspobex *x, void *b, long m, long a, char *s);

//custom attribute set method prototypes
t_max_err mbmspobex_myfloat(t_mbmspobex *x, void *attr, long ac, t_atom *av);
t_max_err mbmspobex_myint(t_mbmspobex *x, void *attr, long ac, t_atom *av);

//other functions
//void mbmspobex_dummy(t_mbmspobex *x);

int main(void)
{
	long attrflags = 0; // no attributes need special flags
	t_class *c;
	t_object *attr;
	
	common_symbols_init();
	
	//Define Class
	c = class_new("mb.mspobex~", (method)mbmspobex_new, (method)mbmspobex_free, sizeof(t_mbmspobex), (method)0L, A_GIMME, 0);
	class_obexoffset_set(c, calcoffset(t_mbmspobex, obex));
	
	// Make methods accessible for our class: 
	class_addmethod(c, (method)mbmspobex_dsp, 		"dsp", A_CANT, 0L);
    //class_addmethod(c, (method)object_obex_dumpout, 	"dumpout", A_CANT,0);  
    class_addmethod(c, (method)object_obex_quickref,	"quickref", A_CANT, 0);
	class_addmethod(c, (method)mbmspobex_assist, 	"assist", A_CANT, 0L);
	
	// Add attributes:
	
	attr = attr_offset_new("myfloat", _sym_float64, attrflags,
						   (method)0L,(method)mbmspobex_myfloat, calcoffset(t_mbmspobex, myfloat));
	attr_addfilter_clip(attr,0., 1., 1, 1); 
	class_addattr(c, attr);	
	
	attr = attr_offset_new("myint", _sym_long, attrflags,
						   (method)0L,(method)mbmspobex_myint, calcoffset(t_mbmspobex, myint));
	attr_addfilter_clip(attr,0, 1, 1, 1); 
	class_addattr(c, attr);	
	
	// Setup class to work with MSP
	class_dspinit(c);
	
	// Finalize class
	class_register(CLASS_BOX, c);
	mbmspobex_class = c;
	
	return 0;
}

// Create
void *mbmspobex_new(t_symbol *s, short argc, t_atom *argv)
{
	t_mbmspobex *x = (t_mbmspobex *)object_alloc(mbmspobex_class);
	if(x){
		//object_obex_store(x, _sym_dumpout, outlet_new(x, NULL));
		
		dsp_setup((t_pxobject *)x, 1);	// Create Object and one signal inlets
		//		x->x_obj.z_misc = Z_NO_INPLACE;  				CHECK		
		outlet_new((t_pxobject *)x, "signal");		// Create a signal Outlet   	
		
		//		x->sr = sys_getsr(); 
		//		x->spms = x->sr / 1000.;
		
		attr_args_process(x, argc, argv); // handle attribute args
	}
	return (x);												// Return the pointer
}

// Destroy
void mbmspobex_free(t_mbmspobex *x)
{
	dsp_free((t_pxobject *)x);		// Always call dsp_free first in this routine
}

void mbmspobex_dsp(t_mbmspobex *x, t_signal **sp, short *count)
{
	//	x->sr = sp[0]->s_sr;
	//	x->spms = x->sr / 1000.;
	
	dsp_add(mbmspobex_perform, 4, x, sp[0]->s_vec, sp[1]->s_vec, sp[0]->s_n);
}

t_max_err mbmspobex_myfloat(t_mbmspobex *x, void *attr, long ac, t_atom *av)
{
	t_box *b;
	float val;
	
	if (ac && av)
	{
		val = atom_getfloat(av);
		
		x->myfloat = val;
		
		object_obex_lookup(x, gensym("#B"), (t_object **)&b);		
	}
	return MAX_ERR_NONE;
}

t_max_err mbmspobex_myint(t_mbmspobex *x, void *attr, long ac, t_atom *av)
{
	t_box *b;
	long val;
	
	if (ac && av)
	{
		val = atom_getlong(av);
		
		x->myint = val;
		
		object_obex_lookup(x, gensym("#B"), (t_object **)&b);
	}
	return MAX_ERR_NONE;
}

t_int *mbmspobex_perform(t_int *w)
{
	t_mbmspobex *x = (t_mbmspobex *)(w[1]); // object is first arg 
	t_float *in = (t_float *)(w[2]);
	t_float *out = (t_float *)(w[3]);
	int n = (int)(w[4]);
	int i;
	//	double output = 0.;
	
	for (i = 0; i < n; i++)
	{
		*out++ = *in++;
	}
	
    return (w + 5); // always add one more than the 2nd argument in dsp_add()
}

void mbmspobex_assist(t_mbmspobex *x, void *b, long m, long a, char *s) // 4 final arguments are always the same for the assistance method
{
	if (m == 1) //input
	{
		switch (a) {	
			case 0:
				sprintf(s,"Audio in (Signal), messages");
				break;
		}
	}
	else
	{
		switch (a) {	
			case 0:
				sprintf(s,"Audio out (Signal)");
				break;
			case 1:
				sprintf(s,"Dump Outlet");
				break;
		}
	}
	
}


